"""Tests for registry module - networks"""
import vcr
from pygbif import registry


@vcr.use_cassette("test/vcr_cassettes/test_networks.yaml")
def test_networks():
    "registry.networks - basic test"
    res = registry.networks()
    assert dict == res.__class__
    assert 2 == len(res)
    assert 4 == len(res["data"])
    assert ["data", "meta"] == sorted(res.keys())


@vcr.use_cassette("test/vcr_cassettes/test_networks_limit.yaml")
def test_networks_limit():
    "registry.networks - limit param "
    res = registry.networks(limit=5)
    assert dict == res.__class__
    assert 4 == len(res["data"])


@vcr.use_cassette("test/vcr_cassettes/test_networks_uuid.yaml")
def test_networks_uuid():
    "registry.networks - with a uuid"
    res = registry.networks(uuid="2b7c7b4f-4d4f-40d3-94de-c28b6fa054a6")
    assert dict == res.__class__
    assert 2 == len(res)
    assert str == res["data"]["title"].__class__
    assert "modifiedBy" in res["data"].keys()



"""Tests for species module - name_suggest methods"""
import vcr
import re
from pygbif import species


@vcr.use_cassette("test/vcr_cassettes/test_name_suggest.yaml")
def test_name_suggest():
    "species.name_suggest - basic test"
    res = species.name_suggest(q="Puma concolor")
    assert list == res.__class__
    assert True == all(
        [bool(re.search("Puma concolor", z["canonicalName"])) for z in res]
    )


@vcr.use_cassette("test/vcr_cassettes/test_name_suggest_paging.yaml")
def test_name_suggest_paging():
    "species.name_suggest - paging"
    res = species.name_suggest(q="Aso", limit=3)
    assert list == res.__class__
    assert 3 == len(res)



"""Tests for registry module - nodes methods"""
import vcr

from pygbif import registry


@vcr.use_cassette("test/vcr_cassettes/test_nodes.yaml")
def test_nodes():
    "registry.nodes - basic test"
    res = registry.nodes()
    assert dict == res.__class__
    assert 2 == len(res)
    assert 100 == len(res["data"])
    assert ["data", "meta"] == sorted(res.keys())


@vcr.use_cassette("test/vcr_cassettes/test_nodes_limit.yaml")
def test_nodes_limit():
    "registry.nodes - limit param"
    res = registry.nodes(limit=5)
    assert dict == res.__class__
    assert 5 == len(res["data"])


@vcr.use_cassette("test/vcr_cassettes/test_nodes_return.yaml")
def test_nodes_return():
    "registry.nodes - data param"
    res = registry.nodes(data="identifier", uuid="03e816b3-8f58-49ae-bc12-4e18b358d6d9")
    assert dict == res.__class__
    assert 2 == len(res)
    assert 1 == len(res["data"])
    assert 5 == len(res["data"][0])
    assert "identifier" in res["data"][0].keys()



"""Tests for species module - name_backbone methods"""
import vcr
from pygbif import species


@vcr.use_cassette("test/vcr_cassettes/test_name_backbone.yaml")
def test_name_backbone():
    "species.name_backbone - basic test"
    res = species.name_backbone(name="Helianthus annuus")
    assert dict == res.__class__
    assert 22 == len(res)
    assert "Helianthus annuus" == res["species"]


@vcr.use_cassette("test/vcr_cassettes/test_name_backbone_multiple_matches.yaml")
def test_name_backbone_multiple_matches():
    "species.name_backbone - multiple matches"
    res = species.name_backbone(name="Aso")
    assert dict == res.__class__
    assert 4 == len(res)
    assert "No match because of too little confidence" == res["note"]



"""
GBIF taxonomic names APIs methods

* `name_backbone`: Lookup names in the GBIF backbone taxonomy
* `name_suggest`: Quick and simple autocomplete lookup service
* `name_usage`: Lookup details for specific names in all taxonomies in GBIF
* `name_lookup`: Lookup names in all taxonomies in GBIF
* `name_parser`: Parse taxon names using the GBIF name parser

If you are looking for behavior similar to the GBIF website when you search
for a name, `name_backbone` may be what you want. For example, a search for
*Lantanophaga pusillidactyla* on the GBIF website and with `name_backbone`
will give back as a first result the correct name
*Lantanophaga pusillidactylus*.
"""

from .name_suggest import name_suggest
from .name_backbone import name_backbone
from .name_lookup import name_lookup
from .name_usage import name_usage
from .name_parser import name_parser



"""
GBIF occurrences APIs methods

* `search`: Search GBIF occurrences
* `get`: Gets details for a single, interpreted occurrence
* `get_verbatim`: Gets a verbatim occurrence record without any interpretation
* `get_fragment`: Get a single occurrence fragment in its raw form (xml or json)
* `count`: Returns occurrence counts for a predefined set of dimensions
* `count_basisofrecord`: Lists occurrence counts by basis of record
* `count_year`: Lists occurrence counts by year
* `count_datasets`: Lists occurrence counts for datasets that cover a given taxon or country
* `count_countries`: Lists occurrence counts for all countries covered by the data published by the given country
* `count_schema`: List the supported metrics by the service
* `count_publishingcountries`: Lists occurrence counts for all countries that publish data about the given country
* `download`: Spin up a download request for GBIF occurrence data
* `download_meta`: Retrieve occurrence download metadata by unique download key
* `download_list`: Lists the downloads created by a user
* `download_get`: Get a download from GBIF
* `download_cancel`: Cancel a download from GBIF
"""

from .search import search
from .get import get, get_verbatim, get_fragment
from .count import (
    count,
    count_basisofrecord,
    count_year,
    count_datasets,
    count_countries,
    count_schema,
    count_publishingcountries,
)
from .download import (
    download,
    download_meta,
    download_list,
    download_get,
    download_cancel,
)



from pygbif.gbifutils import (
    check_data,
    stop,
    gbif_baseurl,
    gbif_GET,
    get_meta,
    parse_results,
    len2,
)


def nodes(
    data="all",
    uuid=None,
    q=None,
    identifier=None,
    identifierType=None,
    limit=100,
    offset=None,
    isocode=None,
    **kwargs
):
    """
  Nodes metadata.

  :param data: [str] The type of data to get. Default: ``all``
  :param uuid: [str] UUID of the data node provider. This must be specified if data
     is anything other than ``all``.
  :param q: [str] Query nodes. Only used when ``data = 'all'``
  :param identifier: [fixnum] The value for this parameter can be a simple string or integer,
      e.g. identifier=120
  :param identifierType: [str] Used in combination with the identifier parameter to filter
      identifiers by identifier type: ``DOI``, ``FTP``, ``GBIF_NODE``, ``GBIF_PARTICIPANT``,
      ``GBIF_PORTAL``, ``HANDLER``, ``LSID``, ``UNKNOWN``, ``URI``, ``URL``, ``UUID``
  :param limit: [int] Number of results to return. Default: ``100``
  :param offset: [int] Record to start at. Default: ``0``
  :param isocode: [str] A 2 letter country code. Only used if ``data = 'country'``.

  :return: A dictionary

  References http://www.gbif.org/developer/registry#nodes

  Usage::

      from pygbif import registry
      registry.nodes(limit=5)
      registry.nodes(identifier=120)
      registry.nodes(uuid="1193638d-32d1-43f0-a855-8727c94299d8")
      registry.nodes(data='identifier', uuid="03e816b3-8f58-49ae-bc12-4e18b358d6d9")
      registry.nodes(data=['identifier','organization','comment'], uuid="03e816b3-8f58-49ae-bc12-4e18b358d6d9")

      uuids = ["8cb55387-7802-40e8-86d6-d357a583c596","02c40d2a-1cba-4633-90b7-e36e5e97aba8",
      "7a17efec-0a6a-424c-b743-f715852c3c1f","b797ce0f-47e6-4231-b048-6b62ca3b0f55",
      "1193638d-32d1-43f0-a855-8727c94299d8","d3499f89-5bc0-4454-8cdb-60bead228a6d",
      "cdc9736d-5ff7-4ece-9959-3c744360cdb3","a8b16421-d80b-4ef3-8f22-098b01a89255",
      "8df8d012-8e64-4c8a-886e-521a3bdfa623","b35cf8f1-748d-467a-adca-4f9170f20a4e",
      "03e816b3-8f58-49ae-bc12-4e18b358d6d9","073d1223-70b1-4433-bb21-dd70afe3053b",
      "07dfe2f9-5116-4922-9a8a-3e0912276a72","086f5148-c0a8-469b-84cc-cce5342f9242",
      "0909d601-bda2-42df-9e63-a6d51847ebce","0e0181bf-9c78-4676-bdc3-54765e661bb8",
      "109aea14-c252-4a85-96e2-f5f4d5d088f4","169eb292-376b-4cc6-8e31-9c2c432de0ad",
      "1e789bc9-79fc-4e60-a49e-89dfc45a7188","1f94b3ca-9345-4d65-afe2-4bace93aa0fe"]

      [ registry.nodes(data='identifier', uuid=x) for x in uuids ]
  """
    args = {
        "q": q,
        "limit": limit,
        "offset": offset,
        "identifier": identifier,
        "identifierType": identifierType,
    }
    data_choices = [
        "all",
        "organization",
        "endpoint",
        "identifier",
        "tag",
        "machineTag",
        "comment",
        "pendingEndorsement",
        "country",
        "dataset",
        "installation",
    ]
    check_data(data, data_choices)

    def getdata(x, uuid, args, **kwargs):
        if x != "all" and uuid is None:
            stop('You must specify a uuid if data does not equal "all"')

        if uuid is None:
            if x == "all":
                url = gbif_baseurl + "node"
            else:
                if isocode is not None and x == "country":
                    url = gbif_baseurl + "node/country/" + isocode
                else:
                    url = gbif_baseurl + "node/" + x
        else:
            if x == "all":
                url = gbif_baseurl + "node/" + uuid
            else:
                url = gbif_baseurl + "node/" + uuid + "/" + x

        res = gbif_GET(url, args, **kwargs)
        return {"meta": get_meta(res), "data": parse_results(res, uuid)}

    # Get data
    if len2(data) == 1:
        return getdata(data, uuid, args, **kwargs)
    else:
        return [getdata(x, uuid, args, **kwargs) for x in data]



import requests_cache
import os.path
import tempfile

try:
    from requests_cache import remove_expired_responses
except ModuleNotFoundError:
    from requests_cache.core import remove_expired_responses

def caching(
    cache=False,
    name=None,
    backend="sqlite",
    expire_after=86400,
    allowable_codes=(200,),
    allowable_methods=("GET",),
):
    """
    pygbif caching management

    :param cache: [bool] if ``True`` all http requests are cached. if ``False`` (default),
        no http requests are cached.
    :param name: [str] the cache name. when backend=sqlite, this is the path for the
        sqlite file, ignored if sqlite not used. if not set, the file is put in your
        temporary directory, and therefore is cleaned up/deleted after closing your
        python session
    :param backend: [str] the backend, one of:

     - ``sqlite`` sqlite database (default)
     - ``memory`` not persistent, stores all data in Python dict in memory
     - ``mongodb`` (experimental) MongoDB database (pymongo < 3.0 required and configured)
     - ``redis`` stores all data on a redis data store (redis required and configured)

    :param expire_after: [str] timedelta or number of seconds after cache will be expired
        or None (default) to ignore expiration. default: 86400 seconds (24 hrs)
    :param allowable_codes: [tuple] limit caching only for response with this codes
        (default: 200)
    :param allowable_methods: [tuple] cache only requests of this methods
        (default: ‘GET’)
    
    :return: sets options to be used by pygbif, returns the options you selected
        in a hash

    Note: setting cache=False will turn off caching, but the backend data still
    persists. thus, you can turn caching back on without losing your cache.
    this also means if you want to delete your cache you have to do it yourself.

    Note: on loading pygbif, we clean up expired responses

    Usage::

        import pygbif
        
        # caching is off by default
        from pygbif import occurrences
        %time z=occurrences.search(taxonKey = 3329049)
        %time w=occurrences.search(taxonKey = 3329049)

        # turn caching on
        pygbif.caching(True)
    
        %time z=occurrences.search(taxonKey = 3329049)
        %time w=occurrences.search(taxonKey = 3329049)

        # set a different backend
        pygbif.caching(cache=True, backend="redis")
        %time z=occurrences.search(taxonKey = 3329049)
        %time w=occurrences.search(taxonKey = 3329049)

        # set a different backend
        pygbif.caching(cache=True, backend="mongodb")
        %time z=occurrences.search(taxonKey = 3329049)
        %time w=occurrences.search(taxonKey = 3329049)
        
        # set path to a sqlite file
        pygbif.caching(name = "some/path/my_file")
    """
    default_name = "pygbif_requests_cache"
    if not cache:
        requests_cache.uninstall_cache()
        CACHE_NAME = None
    else:
        if name is None and backend == "sqlite":
            CACHE_NAME = os.path.join(tempfile.gettempdir(), default_name)
        else:
            CACHE_NAME = default_name

        requests_cache.install_cache(
            cache_name=CACHE_NAME, backend=backend, expire_after=expire_after
        )
        remove_expired_responses()

    cache_settings = {
        "cache": cache,
        "name": CACHE_NAME,
        "backend": backend,
        "expire_after": expire_after,
        "allowable_codes": allowable_codes,
        "allowable_methods": allowable_methods,
    }
    return cache_settings



"""Tests for occurrences module - download_cancle methods"""
from pygbif import occurrences as occ
import vcr


@vcr.use_cassette(
    "test/vcr_cassettes/test_download_cancel.yaml", filter_headers=["authorization"]
)
def test_download_cancel():
    "occurrences.download_cancel - basic test"
    name_key = "156780401"  # for "Bear picornavirus 1"
    res = occ.download("taxonKey = " + name_key)
    download_key = res[0]
    out = occ.download_cancel(download_key)
    assert True == out



"""Tests for species module - name_lookup methods"""
import vcr
from pygbif import species


@vcr.use_cassette("test/vcr_cassettes/test_name_lookup.yaml")
def test_name_lookup():
    "species.name_lookup - basic test"
    res = species.name_lookup(q="mammalia")
    assert dict == res.__class__
    assert 6 == len(res)
    assert 100 == len(res["results"])
    assert "Mammalia" == res["results"][0]["canonicalName"]


@vcr.use_cassette("test/vcr_cassettes/test_name_lookup_paging.yaml")
def test_name_lookup_paging():
    "species.name_lookup - paging"
    res = species.name_lookup(q="mammalia", limit=1)
    assert dict == res.__class__
    assert 6 == len(res)
    assert 1 == len(res["results"])


@vcr.use_cassette("test/vcr_cassettes/test_name_lookup_rank.yaml")
def test_name_lookup_rank():
    "species.name_lookup - rank parameter"
    res = species.name_lookup("Helianthus annuus", rank="species", limit=10)
    assert dict == res.__class__
    assert 10 == len(res["results"])
    assert "SPECIES" == list(set([x["rank"] for x in res["results"]]))[0]


@vcr.use_cassette("test/vcr_cassettes/test_name_lookup_faceting.yaml")
def test_name_lookup_faceting():
    "species.name_lookup - faceting"
    res = species.name_lookup(facet="status", limit=0)
    assert dict == res.__class__
    assert 6 == len(res)
    assert 0 == len(res["results"])
    assert 1 == len(res["facets"])
    assert 2 == len(res["facets"][0])



import time
import requests
import unittest

from pygbif.occurrences.download import GbifDownload, download


class DummyClass(object):
    """A dummy response as given by the requests.post, which can be used
    to mock the posting of requests"""

    text = "0003970-140910143529206"
    status_code = 201


def dummypost(*args, **kwargs):
    """function ot mock the usage of the requests.post functionality"""
    return DummyClass()


class TestGbifClass(unittest.TestCase):
    def test_gbif_creation(self):
        """test the creation of the predicate class"""

        req = GbifDownload("name", "email")
        self.assertIsInstance(req.payload, dict)
        self.assertDictEqual(
            req.payload,
            {
                "created": int(time.strftime("%Y")),
                "creator": "name",
                "notification_address": ["email"],
                "predicate": {"predicates": [], "type": "and"},
                "send_notification": "true",
            },
        )
        self.assertIsNone(req.request_id)

    def test_alternative_main_type(self):
        """test the addition of another predicate combiner"""
        req = GbifDownload("name", "email")
        req.main_pred_type = "or"
        self.assertIsInstance(req.payload, dict)
        self.assertDictEqual(
            req.payload,
            {
                "created": int(time.strftime("%Y")),
                "creator": "name",
                "notification_address": ["email"],
                "predicate": {"predicates": [], "type": "or"},
                "send_notification": "true",
            },
        )

    def test_add_predicate(self):
        """test the predicate addition"""
        req = GbifDownload("name", "email")
        req.add_predicate("COUNTRY", "BE", "equals")
        self.assertIsInstance(req.payload["predicate"]["predicates"], list)
        self.assertEqual(len(req.payload["predicate"]["predicates"]), 1)
        self.assertIsInstance(req.payload["predicate"]["predicates"][0], dict)
        self.assertDictEqual(
            req.payload["predicate"]["predicates"][0],
            {"key": "COUNTRY", "type": "equals", "value": "BE"},
        )

    def test_add_iterative_predicate(self):
        """the the predicate addition of an iterative sequence"""
        req = GbifDownload("name", "email")
        req.add_iterative_predicate("TAXONKEY", [3189866, 2498252])
        self.assertIsInstance(req.payload["predicate"]["predicates"], list)
        self.assertEqual(len(req.payload["predicate"]["predicates"]), 1)
        self.assertIsInstance(req.payload["predicate"]["predicates"][0], dict)

        temp_pred = req.payload["predicate"]["predicates"][0]["predicates"]
        self.assertIsInstance(temp_pred, list)
        self.assertEqual(len(temp_pred), 2)
        self.assertIsInstance(temp_pred[0], dict)

        self.assertEqual(set(list(temp_pred[0].keys())), set(["key", "type", "value"]))
        self.assertEqual(req.payload["predicate"]["predicates"][0]["type"], "or")

    def test_add_geometry(self):
        """check predicate after adding a geometry"""
        req = GbifDownload("name", "email")
        req.add_geometry(
            "POLYGON((-14.06 42.55, 9.84 38.27, -7.03 26.43, -14.06 42.55))"
        )
        self.assertIsInstance(req.payload["predicate"]["predicates"], list)
        self.assertEqual(len(req.payload["predicate"]["predicates"]), 1)
        self.assertEqual(
            set(req.payload["predicate"]["predicates"][0].keys()),
            set(["type", "geometry"]),
        )

    def test_lookup_predicates(self):
        """different lookups checked"""
        # main combination predicate
        req = GbifDownload("name", "email")
        req.main_pred_type = "|"
        self.assertEqual(req.payload["predicate"]["type"], "or")

        # predicate addition
        req.add_predicate("YEAR", "2000", ">=")
        self.assertEqual(
            req.payload["predicate"]["predicates"][0]["type"], "greaterThanOrEquals"
        )

    # mocking the request service
    requests.post = dummypost

    def test_post_download(self):
        req = GbifDownload("name", "email")
        req.add_iterative_predicate(
            "BASIS_OF_RECORD", ["FOSSIL_SPECIMEN", "LITERATURE"]
        )
        dl_key = req.post_download("name", "pwd")
        if not dl_key:
            raise KeyError(
                "You might have too many downloads running at the \
                            same time. Check your downloads page!"
            )

        while req.get_status() in ["PREPARING", "RUNNING"]:
            print("Preparing ...")
            time.sleep(10)
        self.assertIn(req.get_status(), ["SUCCEEDED", "KILLED"])


class TestDownload(unittest.TestCase):

    # mocking the request service
    requests.post = dummypost

    def test_single_predicate(self):
        dl_key, payload = download(
            "decimalLatitude > 50", user="dummy", email="dummy", pwd="dummy"
        )
        self.assertDictEqual(
            payload["predicate"]["predicates"][0],
            {"key": "DECIMAL_LATITUDE", "type": "greaterThan", "value": "50"},
        )

        dl_key, payload = download(
            "basisOfRecord = LITERATURE", user="dummy", email="dummy", pwd="dummy"
        )
        self.assertDictEqual(
            payload["predicate"]["predicates"][0],
            {"key": "BASIS_OF_RECORD", "type": "equals", "value": "LITERATURE"},
        )

    def test_single_predicate_list(self):
        dl_key, payload = download(
            ["decimalLatitude > 50"], user="dummy", email="dummy", pwd="dummy"
        )

        self.assertDictEqual(
            payload["predicate"]["predicates"][0],
            {"key": "DECIMAL_LATITUDE", "type": "greaterThan", "value": "50"},
        )

    def test_multiple_predicates(self):
        dl_key, payload = download(
            ["taxonKey = 7264332", "hasCoordinate = TRUE"],
            user="dummy",
            email="dummy",
            pwd="dummy",
        )
        temp_pred = payload["predicate"]["predicates"]
        self.assertIsInstance(temp_pred, list)
        self.assertEqual(len(temp_pred), 2)
        self.assertIsInstance(temp_pred[0], dict)
        self.assertIsInstance(temp_pred[1], dict)
        self.assertEqual(set(list(temp_pred[0].keys())), set(["key", "type", "value"]))
        self.assertEqual(set(list(temp_pred[1].keys())), set(["key", "type", "value"]))

    def test_alternative_main_type(self):
        dl_key, payload = download(
            ["depth = 80", "taxonKey = 2343454"],
            pred_type="or",
            user="dummy",
            email="dummy",
            pwd="dummy",
        )

        self.assertEqual(payload["predicate"]["type"], "or")

    def test_geometry_predicate(self):
        dl_key, payload = download(
            ["geometry within POLYGON((-82.7 36.9, -85.0 35.6, -81.0 33.5, -79.4 36.3, -79.4 36.3, -82.7 36.9))"], 
            user="dummy", email="dummy", pwd="dummy"
        )

        self.assertDictEqual(
            payload["predicate"]["predicates"][0],
            {"type": "within", "geometry": "POLYGON((-82.7 36.9, -85.0 35.6, -81.0 33.5, -79.4 36.3, -79.4 36.3, -82.7 36.9))"},
        )



from pygbif.gbifutils import (
    check_data,
    stop,
    gbif_baseurl,
    gbif_GET,
    get_meta,
    parse_results,
    len2,
)


def installations(
    data="all",
    uuid=None,
    q=None,
    identifier=None,
    identifierType=None,
    limit=100,
    offset=None,
    **kwargs
):
    """
  Installations metadata.

  :param data: [str] The type of data to get. Default is all data. If not ``all``, then one
     or more of ``contact``, ``endpoint``, ``dataset``, ``comment``, ``deleted``, ``nonPublishing``.
  :param uuid: [str] UUID of the data node provider. This must be specified if data
     is anything other than ``all``.
  :param q: [str] Query nodes. Only used when ``data='all'``. Ignored otherwise.
  :param identifier: [fixnum] The value for this parameter can be a simple string or integer,
      e.g. identifier=120
  :param identifierType: [str] Used in combination with the identifier parameter to filter
      identifiers by identifier type: ``DOI``, ``FTP``, ``GBIF_NODE``, ``GBIF_PARTICIPANT``,
      ``GBIF_PORTAL``, ``HANDLER``, ``LSID``, ``UNKNOWN``, ``URI``, ``URL``, ``UUID``
  :param limit: [int] Number of results to return. Default: ``100``
  :param offset: [int] Record to start at. Default: ``0``

  :return: A dictionary

  References: http://www.gbif.org/developer/registry#installations

  Usage::

      from pygbif import registry
      registry.installations(limit=5)
      registry.installations(q="france")
      registry.installations(uuid="b77901f9-d9b0-47fa-94e0-dd96450aa2b4")
      registry.installations(data='contact', uuid="b77901f9-d9b0-47fa-94e0-dd96450aa2b4")
      registry.installations(data='contact', uuid="2e029a0c-87af-42e6-87d7-f38a50b78201")
      registry.installations(data='endpoint', uuid="b77901f9-d9b0-47fa-94e0-dd96450aa2b4")
      registry.installations(data='dataset', uuid="b77901f9-d9b0-47fa-94e0-dd96450aa2b4")
      registry.installations(data='deleted')
      registry.installations(data='deleted', limit=2)
      registry.installations(data=['deleted','nonPublishing'], limit=2)
      registry.installations(identifierType='DOI', limit=2)
  """
    args = {
        "q": q,
        "limit": limit,
        "offset": offset,
        "identifier": identifier,
        "identifierType": identifierType,
    }
    data_choices = [
        "all",
        "contact",
        "endpoint",
        "dataset",
        "identifier",
        "tag",
        "machineTag",
        "comment",
        "deleted",
        "nonPublishing",
    ]
    check_data(data, data_choices)

    def getdata(x, uuid, args, **kwargs):
        if x not in ["all", "deleted", "nonPublishing"] and uuid is None:
            stop(
                "You must specify a uuid if data does not equal all and data does not equal one of deleted or nonPublishing"
            )

        if uuid is None:
            if x == "all":
                url = gbif_baseurl + "installation"
            else:
                url = gbif_baseurl + "installation/" + x
        else:
            if x == "all":
                url = gbif_baseurl + "installation/" + uuid
            else:
                url = gbif_baseurl + "installation/" + uuid + "/" + x

        res = gbif_GET(url, args, **kwargs)
        return {"meta": get_meta(res), "data": parse_results(res, uuid)}

    if len2(data) == 1:
        return getdata(data, uuid, args, **kwargs)
    else:
        return [getdata(x, uuid, args, **kwargs) for x in data]



from pygbif.gbifutils import gbif_baseurl, gbif_GET


def name_suggest(q=None, datasetKey=None, rank=None, limit=100, offset=None, **kwargs):
    """
  A quick and simple autocomplete service that returns up to 20 name usages by
  doing prefix matching against the scientific name. Results are ordered by relevance.

  :param q: [str] Simple search parameter. The value for this parameter can be a
     simple word or a phrase. Wildcards can be added to the simple word parameters only,
     e.g. ``q=*puma*`` (Required)
  :param datasetKey: [str] Filters by the checklist dataset key (a uuid, see examples)
  :param rank: [str] A taxonomic rank. One of ``class``, ``cultivar``, ``cultivar_group``, ``domain``, ``family``,
     ``form``, ``genus``, ``informal``, ``infrageneric_name``, ``infraorder``, ``infraspecific_name``,
     ``infrasubspecific_name``, ``kingdom``, ``order``, ``phylum``, ``section``, ``series``, ``species``, ``strain``, ``subclass``,
     ``subfamily``, ``subform``, ``subgenus``, ``subkingdom``, ``suborder``, ``subphylum``, ``subsection``, ``subseries``,
     ``subspecies``, ``subtribe``, ``subvariety``, ``superclass``, ``superfamily``, ``superorder``, ``superphylum``,
     ``suprageneric_name``, ``tribe``, ``unranked``, or ``variety``.
  :param limit: [fixnum] Number of records to return. Maximum: ``1000``. (optional)
  :param offset: [fixnum] Record number to start at. (optional)

  :return: A dictionary

  References: http://www.gbif.org/developer/species#searching

  Usage::

      from pygbif import species

      species.name_suggest(q='Puma concolor')
      x = species.name_suggest(q='Puma')
      species.name_suggest(q='Puma', rank="genus")
      species.name_suggest(q='Puma', rank="subspecies")
      species.name_suggest(q='Puma', rank="species")
      species.name_suggest(q='Puma', rank="infraspecific_name")
      species.name_suggest(q='Puma', limit=2)
  """
    url = gbif_baseurl + "species/suggest"
    args = {"q": q, "rank": rank, "offset": offset, "limit": limit}
    return gbif_GET(url, args, **kwargs)


def suggestfields():
    """
  Fields available in ``gbif_suggest()`` function
  """
    return [
        "key",
        "datasetTitle",
        "datasetKey",
        "nubKey",
        "parentKey",
        "parent",
        "kingdom",
        "phylum",
        "class",
        "order",
        "family",
        "genus",
        "species",
        "kingdomKey",
        "phylumKey",
        "classKey",
        "orderKey",
        "familyKey",
        "genusKey",
        "speciesKey",
        "species",
        "canonicalName",
        "authorship",
        "accordingTo",
        "nameType",
        "taxonomicStatus",
        "rank",
        "numDescendants",
        "numOccurrences",
        "sourceId",
        "nomenclaturalStatus",
        "threatStatuses",
        "synonym",
        "higherClassificationMap",
    ]



"""
GBIF registry APIs methods

* `organizations`: Organizations metadata
* `nodes`: Nodes metadata
* `networks`: Networks metadata
* `installations`: Installations metadata
* `datasets`: Search for datasets and dataset metadata
* `dataset_metrics`: Get details/metrics on a GBIF dataset
* `dataset_suggest`: Search that returns up to 20 matching datasets
* `dataset_search`: Full text search across all datasets
"""

from .nodes import nodes
from .networks import networks
from .installations import installations
from .datasets import datasets, dataset_metrics, dataset_suggest, dataset_search
from .organizations import organizations



"""Tests for utils module - wkt_rewind"""
import pytest
import unittest
import re
from pygbif import utils

x = "POLYGON((144.6 13.2, 144.6 13.6, 144.9 13.6, 144.9 13.2, 144.6 13.2))"


def test_wkt_rewind():
    "utils.wkt_rewind - basic test"
    res = utils.wkt_rewind(x)
    assert str == res.__class__


class TestWKTClass(unittest.TestCase):
    def test_wkt_rewind_digits(self):
        "utils.wkt_rewind - digits works as expected"
        res = utils.wkt_rewind(x, digits=3)
        pat = re.compile("\\.[0-9]{3}\\s")
        out = pat.search(res)
        self.assertIsInstance(out.string, str)


def test_wkt_rewind_fails_well():
    "utils.wkt_rewind - fails well"
    with pytest.raises(TypeError):
        utils.wkt_rewind(x, digits="foo")



from pygbif.gbifutils import gbif_baseurl, gbif_GET


def get(key, **kwargs):
    """
    Gets details for a single, interpreted occurrence

    :param key: [int] A GBIF occurrence key

    :return: A dictionary, of results

    Usage::

        from pygbif import occurrences
        occurrences.get(key = 1258202889)
        occurrences.get(key = 1227768771)
        occurrences.get(key = 1227769518)
    """
    url = gbif_baseurl + "occurrence/" + str(key)
    out = gbif_GET(url, {}, **kwargs)
    return out


def get_verbatim(key, **kwargs):
    """
    Gets a verbatim occurrence record without any interpretation

    :param key: [int] A GBIF occurrence key

    :return: A dictionary, of results

    Usage::

        from pygbif import occurrences
        occurrences.get_verbatim(key = 1258202889)
        occurrences.get_verbatim(key = 1227768771)
        occurrences.get_verbatim(key = 1227769518)
    """
    url = gbif_baseurl + "occurrence/" + str(key) + "/verbatim"
    out = gbif_GET(url, {}, **kwargs)
    return out


def get_fragment(key, **kwargs):
    """
    Get a single occurrence fragment in its raw form (xml or json)

    :param key: [int] A GBIF occurrence key

    :return: A dictionary, of results

    Usage::

        from pygbif import occurrences
        occurrences.get_fragment(key = 1052909293)
        occurrences.get_fragment(key = 1227768771)
        occurrences.get_fragment(key = 1227769518)
    """
    url = gbif_baseurl + "occurrence/" + str(key) + "/fragment"
    out = gbif_GET(url, {}, **kwargs)
    return out



import os
import csv
import json
import re
import datetime
import requests

from .. import package_metadata, occurrences
from ..gbifutils import (
    is_not_none,
    is_none,
    stop,
    gbif_GET,
    gbif_GET_write,
    gbif_DELETE,
)
import logging


def _parse_args(x):
    if re.match("geometry", x):
        geometry = re.search("(POLY|MULTIPOLY).+", x, re.IGNORECASE).group()
        return {"type": "within", "geometry": geometry}
    tmp = re.split("\s", x)
    pred_type = operator_lkup.get(tmp[1])
    key = key_lkup.get(tmp[0])
    return {"type": pred_type, "key": key, "value": tmp[2]}


def _check_environ(variable, value):
    """check if a variable is present in the environmental variables"""
    if is_not_none(value):
        return value
    else:
        value = os.environ.get(variable)
        if is_none(value):
            stop(
                "".join(
                    [
                        variable,
                        """ not supplied and no entry in environmental
                           variables""",
                    ]
                )
            )
        else:
            return value


def download(queries, user=None, pwd=None, email=None, pred_type="and"):
    """
    Spin up a download request for GBIF occurrence data.

    :param queries: One or more of query arguments to kick of a download job.
        See Details.
    :type queries: str or list
    :param pred_type: (character) One of ``equals`` (``=``), ``and`` (``&``),
        `or`` (``|``), ``lessThan`` (``<``), ``lessThanOrEquals`` (``<=``),
        ``greaterThan`` (``>``), ``greaterThanOrEquals`` (``>=``),
        ``in``, ``within``, ``not`` (``!``), ``like``
    :param user: (character) User name within GBIF's website.
        Required. Set in your env vars with the option ``GBIF_USER``
    :param pwd: (character) User password within GBIF's website. Required.
        Set in your env vars with the option ``GBIF_PWD``
    :param email: (character) Email address to recieve download notice done
        email. Required. Set in your env vars with the option ``GBIF_EMAIL``

    Argument passed have to be passed as character (e.g., ``country = US``),
    with a space between key (``country``), operator (``=``), and value (``US``).
    See the ``type`` parameter for possible options for the operator.
    This character string is parsed internally.

    Acceptable arguments to ``...`` (args) are:

     - taxonKey = ``TAXON_KEY``
     - scientificName = ``SCIENTIFIC_NAME``
     - country = ``COUNTRY``
     - publishingCountry = ``PUBLISHING_COUNTRY``
     - hasCoordinate = ``HAS_COORDINATE``
     - hasGeospatialIssue = ``HAS_GEOSPATIAL_ISSUE``
     - typeStatus = ``TYPE_STATUS``
     - recordNumber = ``RECORD_NUMBER``
     - lastInterpreted = ``LAST_INTERPRETED``
     - continent = ``CONTINENT``
     - geometry = ``GEOMETRY``
     - basisOfRecord = ``BASIS_OF_RECORD``
     - datasetKey = ``DATASET_KEY``
     - eventDate = ``EVENT_DATE``
     - catalogNumber = ``CATALOG_NUMBER``
     - year = ``YEAR``
     - month = ``MONTH``
     - decimalLatitude = ``DECIMAL_LATITUDE``
     - decimalLongitude = ``DECIMAL_LONGITUDE``
     - elevation = ``ELEVATION``
     - depth = ``DEPTH``
     - institutionCode = ``INSTITUTION_CODE``
     - collectionCode = ``COLLECTION_CODE``
     - issue = ``ISSUE``
     - mediatype = ``MEDIA_TYPE``
     - recordedBy = ``RECORDED_BY``
     - repatriated = ``REPATRIATED``

    See the API docs http://www.gbif.org/developer/occurrence#download
    for more info, and the predicates docs
    http://www.gbif.org/developer/occurrence#predicates

    GBIF has a limit of 12,000 characters for download queries - so
    if you're download request is really, really long and complex,
    consider breaking it up into multiple requests by one factor or
    another.

    :return: A dictionary, of results

    Usage::

        from pygbif import occurrences as occ

        occ.download('basisOfRecord = PRESERVED_SPECIMEN')
        occ.download('taxonKey = 3119195')
        occ.download('decimalLatitude > 50')
        occ.download('elevation >= 9000')
        occ.download('decimalLatitude >= 65')
        occ.download('country = US')
        occ.download('institutionCode = TLMF')
        occ.download('catalogNumber = Bird.27847588')

        res = occ.download(['taxonKey = 7264332', 'hasCoordinate = TRUE'])

        # pass output to download_meta for more information
        occ.download_meta(occ.download('decimalLatitude > 75'))

        # Multiple queries
        gg = occ.download(['decimalLatitude >= 65',
                          'decimalLatitude <= -65'], type='or')
        gg = occ.download(['depth = 80', 'taxonKey = 2343454'],
                          type='or')

        # Repratriated data for Costa Rica
        occ.download(['country = CR', 'repatriated = true'])

        # turn off logging
        import logging
        logger = logging.getLogger()
        logger.disabled = True
        z = occ.download('elevation >= 95000')
        logger.disabled = False
        w = occ.download('elevation >= 10000')
    """

    user = _check_environ("GBIF_USER", user)
    pwd = _check_environ("GBIF_PWD", pwd)
    email = _check_environ("GBIF_EMAIL", email)

    if isinstance(queries, str):
        queries = [queries]

    keyval = [_parse_args(z) for z in queries]

    # USE GBIFDownload class to set up the predicates
    req = GbifDownload(user, email)
    req.main_pred_type = pred_type
    for predicate in keyval:
        if "geometry" in predicate.keys():
            req.add_geometry(predicate["geometry"])
        else:
            req.add_predicate(predicate["key"], predicate["value"], predicate["type"])

    out = req.post_download(user, pwd)
    return out, req.payload


class GbifDownload(object):
    def __init__(self, creator, email, polygon=None):
        """class to setup a JSON doc with the query and POST a request

        All predicates (default key-value or iterative based on a list of
        values) are combined with an AND statement. Iterative predicates are
        creating a subset equal statements combined with OR

        :param creator: User name.
        :param email: user email
        :param polygon: Polygon of points to extract data from
        """
        self.predicates = []
        self._main_pred_type = "and"

        self.url = "http://api.gbif.org/v1/occurrence/download/request"
        self.header = {
            "accept": "application/json",
            "content-type": "application/json",
            "user-agent": "".join(
                [
                    "python-requests/",
                    requests.__version__,
                    ",pygbif/",
                    package_metadata.__version__,
                ]
            ),
        }

        self.payload = {
            "creator": creator,
            "notification_address": [email],
            "send_notification": "true",
            "created": datetime.date.today().year,
            "predicate": {"type": self._main_pred_type, "predicates": self.predicates},
        }
        self.request_id = None

        # prepare the geometry polygon constructions
        if polygon:
            self.add_geometry(polygon)

    @property
    def main_pred_type(self):
        """get main predicate combination type"""
        return self._main_pred_type

    @main_pred_type.setter
    def main_pred_type(self, value):
        """set main predicate combination type

        :param value: (character) One of ``equals`` (``=``), ``and`` (``&``), ``or`` (``|``),
        ``lessThan`` (``<``), ``lessThanOrEquals`` (``<=``), ``greaterThan`` (``>``),
        ``greaterThanOrEquals`` (``>=``), ``in``, ``within``, ``not`` (``!``), ``like``
        """
        if value not in operators:
            value = operator_lkup.get(value)
        if value:
            self._main_pred_type = value
            self.payload["predicate"]["type"] = self._main_pred_type
        else:
            raise Exception("main predicate combiner not a valid operator")

    def add_predicate(self, key, value, predicate_type="equals"):
        """
        add key, value, type combination of a predicate

        :param key: query KEY parameter
        :param value: the value used in the predicate
        :param predicate_type: the type of predicate (e.g. ``equals``)
        """
        if predicate_type not in operators:
            predicate_type = operator_lkup.get(predicate_type)
        if predicate_type:
            self.predicates.append({"type": predicate_type, "key": key, "value": value})
        else:
            raise Exception("predicate type not a valid operator")

    @staticmethod
    def _extract_values(values_list):
        """extract values from either file or list

        :param values_list: list or file name (str) with list of values
        """
        values = []
        # check if file or list of values to iterate
        if isinstance(values_list, str):
            with open(values_list) as ff:
                reading = csv.reader(ff)
                for j in reading:
                    values.append(j[0])
        elif isinstance(values_list, list):
            values = values_list
        else:
            raise Exception("input datatype not supported.")
        return values

    def add_iterative_predicate(self, key, values_list):
        """add an iterative predicate with a key and set of values
        which it can be equal to in and or function.
        The individual predicates are specified with the type ``equals`` and
        combined with a type ``or``.

        The main reason for this addition is the inability of using ``in`` as
        predicate type wfor multiple taxon_key values
        (cfr. http://dev.gbif.org/issues/browse/POR-2753)

        :param key: API key to use for the query.
        :param values_list: Filename or list containing the taxon keys to be s
            searched.

        """
        values = self._extract_values(values_list)

        predicate = {"type": "equals", "key": key, "value": None}
        predicates = []
        while values:
            predicate["value"] = values.pop()
            predicates.append(predicate.copy())
        self.predicates.append({"type": "or", "predicates": predicates})

    def add_geometry(self, polygon, geom_type="within"):
        """add a geometry type of predicate

        :param polygon: In this format ``POLYGON((x1 y1, x2 y2,... xn yn))``
        :param geom_type: type of predicate, e.g. ``within``
        :return:
        """
        self.predicates.append({"type": geom_type, "geometry": polygon})

    def post_download(self, user=None, pwd=None):
        """

        :param user: Username
        :param pwd: Password
        :return:
        """
        user = _check_environ("GBIF_USER", user)
        pwd = _check_environ("GBIF_PWD", pwd)

        # pprint.pprint(self.payload)
        r = requests.post(
            self.url,
            auth=requests.auth.HTTPBasicAuth(user, pwd),
            data=json.dumps(self.payload),
            headers=self.header,
        )
        if r.status_code > 203:
            raise Exception(
                "error: "
                + r.text
                + ", with error status code "
                + str(r.status_code)
                + "check your number of active downloads."
            )
        else:
            self.request_id = r.text
            logging.info("Your download key is " + self.request_id)
        return self.request_id

    def get_status(self):
        """get the current download status"""
        return get_download_status(self.request_id)


def get_download_status(request_key):
    """get the current download status"""
    return download_meta(request_key).get("status")


def download_meta(key, **kwargs):
    """
    Retrieves the occurrence download metadata by its unique key. Further
    named arguments passed on to ``requests.get`` can be included as additional
    arguments

    :param key: [str] A key generated from a request, like that from ``download``

    Usage::

      from pygbif import occurrences as occ
      occ.download_meta(key = "0003970-140910143529206")
      occ.download_meta(key = "0000099-140929101555934")
    """
    url = "http://api.gbif.org/v1/occurrence/download/" + key
    return gbif_GET(url, {}, **kwargs)


def download_cancel(key, user=None, pwd=None, **kwargs):
    """
    Delete a download request by its unique key. Further
    named arguments passed on to ``requests.get`` can be included as additional
    arguments

    :param key: [str] A key generated from a request, like that from ``download``
    :param user: [str] A user name, look at env var ``GBIF_USER`` first
    :param pwd: [str] Your password, look at env var ``GBIF_PWD`` first

    :return: a bool, `True` if cancel request successful, otherwise `False`

    Usage::

      from pygbif import occurrences as occ
      # first, make a download request
      x = occ.download('taxonKey = 156780401')
      occ.download_meta(x[0])
      # then cancel it - do so before the download is ready, or it will have no effect
      occ.download_cancel(key = x[0])
    """
    user = _check_environ("GBIF_USER", user)
    pwd = _check_environ("GBIF_PWD", pwd)

    url = "http://api.gbif.org/v1/occurrence/download/request/" + key
    return gbif_DELETE(url, {}, auth=(user, pwd), **kwargs)


def download_list(user=None, pwd=None, limit=20, offset=0):
    """
    Lists the downloads created by a user.

    :param user: [str] A user name, look at env var ``GBIF_USER`` first
    :param pwd: [str] Your password, look at env var ``GBIF_PWD`` first
    :param limit: [int] Number of records to return. Default: ``20``
    :param offset: [int] Record number to start at. Default: ``0``

    Usage::

      from pygbif import occurrences as occ
      occ.download_list(user = "sckott")
      occ.download_list(user = "sckott", limit = 5)
      occ.download_list(user = "sckott", offset = 21)
    """

    user = _check_environ("GBIF_USER", user)
    pwd = _check_environ("GBIF_PWD", pwd)

    url = "http://api.gbif.org/v1/occurrence/download/user/" + user
    args = {"limit": limit, "offset": offset}
    res = gbif_GET(url, args, auth=(user, pwd))
    return {
        "meta": {
            "offset": res["offset"],
            "limit": res["limit"],
            "endofrecords": res["endOfRecords"],
            "count": res["count"],
        },
        "results": res["results"],
    }


def download_get(key, path=".", **kwargs):
    """
    Get a download from GBIF.

    :param key: [str] A key generated from a request, like that from ``download``
    :param path: [str] Path to write zip file to. Default: ``"."``, with a ``.zip`` appended to the end.
    :param **kwargs**: Further named arguments passed on to ``requests.get``

    Downloads the zip file to a directory you specify on your machine.
    The speed of this function is of course proportional to the size of the
    file to download, and affected by your internet connection speed.

    This function only downloads the file. To open and read it, see
    https://github.com/BelgianBiodiversityPlatform/python-dwca-reader

    Usage::

      from pygbif import occurrences as occ
      x=occ.download_get("0000066-140928181241064")
      occ.download_get("0003983-140910143529206")
      
      # turn off logging
      import logging
      logger = logging.getLogger()
      logger.disabled = True
      x = occ.download_get("0000066-140928181241064")
      
      # turn back on
      logger.disabled = False
      x = occ.download_get("0000066-140928181241064")
    """
    meta = occurrences.download_meta(key)
    if meta["status"] != "SUCCEEDED":
        raise Exception('download "%s" not of status SUCCEEDED' % key)
    else:
        logging.info("Download file size: %s bytes" % meta["size"])
        url = "http://api.gbif.org/v1/occurrence/download/request/" + key
        path = "%s/%s.zip" % (path, key)
        gbif_GET_write(url, path, **kwargs)
        logging.info("On disk at " + path)
        return {"path": path, "size": meta["size"], "key": key}


operators = [
    "equals",
    "and",
    "or",
    "lessThan",
    "lessThanOrEquals",
    "greaterThan",
    "greaterThanOrEquals",
    "in",
    "within",
    "not",
    "like",
]

operator_lkup = {
    "=": "equals",
    "&": "and",
    "|": "or",
    "<": "lessThan",
    "<=": "lessThanOrEquals",
    ">": "greaterThan",
    ">=": "greaterThanOrEquals",
    "!": "not",
    "in": "in",
    "within": "within",
    "like": "like",
}

key_lkup = {
    "taxonKey": "TAXON_KEY",
    "scientificName": "SCIENTIFIC_NAME",
    "country": "COUNTRY",
    "publishingCountry": "PUBLISHING_COUNTRY",
    "hasCoordinate": "HAS_COORDINATE",
    "hasGeospatialIssue": "HAS_GEOSPATIAL_ISSUE",
    "typeStatus": "TYPE_STATUS",
    "recordNumber": "RECORD_NUMBER",
    "lastInterpreted": "LAST_INTERPRETED",
    "continent": "CONTINENT",
    "geometry": "GEOMETRY",
    "basisOfRecord": "BASIS_OF_RECORD",
    "datasetKey": "DATASET_KEY",
    "eventDate": "EVENT_DATE",
    "catalogNumber": "CATALOG_NUMBER",
    "year": "YEAR",
    "month": "MONTH",
    "decimalLatitude": "DECIMAL_LATITUDE",
    "decimalLongitude": "DECIMAL_LONGITUDE",
    "elevation": "ELEVATION",
    "depth": "DEPTH",
    "institutionCode": "INSTITUTION_CODE",
    "collectionCode": "COLLECTION_CODE",
    "issue": "ISSUE",
    "mediatype": "MEDIA_TYPE",
    "recordedBy": "RECORDED_BY",
    "repatriated": "REPATRIATED",
}



import codecs
from setuptools import setup
from setuptools import find_packages

with codecs.open("README.rst", "r", "utf-8") as f:
    readme = f.read().replace("\r", '')

with codecs.open("Changelog.rst", "r", "utf-8") as f:
    changes = f.read().replace("\r", '')
changes = changes.replace(":issue:", "")
long_description = readme + "\n\n" + changes

setup(
    name="pygbif",
    version="0.6.1",
    description="Python client for GBIF",
    long_description=long_description,
    long_description_content_type="text/x-rst",
    author="Scott Chamberlain",
    author_email="myrmecocystus@gmail.com",
    url="http://github.com/gbif/pygbif",
    download_url="https://github.com/gbif/pygbif/archive/refs/tags/v0.6.1.tar.gz",
    license="MIT",
    packages=find_packages(exclude=["test-*"]),
    install_requires=[
        "requests>2.7",
        "requests-cache",
        "geojson_rewind",
        "geomet",
        "appdirs>=1.4.3",
        "matplotlib",
    ],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
        "Natural Language :: English",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9"
    ],
    keywords = ['gbif', 'biodiversity', 'specimens', 'API', 'web-services', 'occurrences', 'species', 'taxonomy'],
)



from pygbif.gbifutils import gbif_baseurl, bool2str, gbif_GET


def name_backbone(
    name,
    rank=None,
    kingdom=None,
    phylum=None,
    clazz=None,
    order=None,
    family=None,
    genus=None,
    strict=False,
    verbose=False,
    offset=None,
    limit=100,
    **kwargs
):
    """
  Lookup names in the GBIF backbone taxonomy.

  :param name: [str] Full scientific name potentially with authorship (required)
  :param rank: [str] The rank given as our rank enum. (optional)
  :param kingdom: [str] If provided default matching will also try to match against this
     if no direct match is found for the name alone. (optional)
  :param phylum: [str] If provided default matching will also try to match against this
     if no direct match is found for the name alone. (optional)
  :param class: [str] If provided default matching will also try to match against this
     if no direct match is found for the name alone. (optional)
  :param order: [str] If provided default matching will also try to match against this
     if no direct match is found for the name alone. (optional)
  :param family: [str] If provided default matching will also try to match against this
     if no direct match is found for the name alone. (optional)
  :param genus: [str] If provided default matching will also try to match against this
     if no direct match is found for the name alone. (optional)
  :param strict: [bool] If ``True`` it (fuzzy) matches only the given name, but never a
     taxon in the upper classification (optional)
  :param verbose: [bool] If ``True`` show alternative matches considered which had been rejected.
  :param offset: [int] Record to start at. Default: ``0``
  :param limit: [int] Number of results to return. Default: ``100``

  If you are looking for behavior similar to the GBIF website when you search
  for a name, `name_backbone` may be what you want. For example, a search for
  *Lantanophaga pusillidactyla* on the GBIF website and with `name_backbone`
  will give back as a first result the correct name
  *Lantanophaga pusillidactylus*.

  A list for a single taxon with many slots (with ``verbose=False`` - default), or a
  list of length two, first element for the suggested taxon match, and a data.frame
  with alternative name suggestions resulting from fuzzy matching (with ``verbose=True``).

  If you don't get a match GBIF gives back a list of length 3 with slots synonym,
  confidence, and ``matchType='NONE'``.

  reference: https://www.gbif.org/developer/species#searching

  Usage::

      from pygbif import species
      species.name_backbone(name='Helianthus annuus', kingdom='plants')
      species.name_backbone(name='Helianthus', rank='genus', kingdom='plants')
      species.name_backbone(name='Poa', rank='genus', family='Poaceae')

      # Verbose - gives back alternatives
      species.name_backbone(name='Helianthus annuus', kingdom='plants', verbose=True)

      # Strictness
      species.name_backbone(name='Poa', kingdom='plants', verbose=True, strict=False)
      species.name_backbone(name='Helianthus annuus', kingdom='plants', verbose=True, strict=True)

      # Non-existent name
      species.name_backbone(name='Aso')

      # Multiple equal matches
      species.name_backbone(name='Oenante')
  """
    url = gbif_baseurl + "species/match"
    args = {
        "name": name,
        "rank": rank,
        "kingdom": kingdom,
        "phylum": phylum,
        "class": clazz,
        "order": order,
        "family": family,
        "genus": genus,
        "strict": bool2str(strict),
        "verbose": bool2str(verbose),
        "offset": offset,
        "limit": limit,
    }
    tt = gbif_GET(url, args, **kwargs)
    return tt



"""Tests for registry module - dataset_metrics method"""
import vcr
from pygbif import registry


@vcr.use_cassette("test/vcr_cassettes/test_dataset_metrics.yaml")
def test_dataset_metrics():
    "registry.dataset_metrics - basic test"
    res = registry.dataset_metrics(uuid="3f8a1297-3259-4700-91fc-acc4170b27ce")
    assert dict == res.__class__
    assert 19 == len(res)


@vcr.use_cassette("test/vcr_cassettes/test_dataset_metrics_multiple_uuids.yaml")
def test_dataset_metrics_multiple_uuids():
    "registry.dataset_metrics - multiple uuids"
    uuids = [
        "3f8a1297-3259-4700-91fc-acc4170b27ce",
        "66dd0960-2d7d-46ee-a491-87b9adcfe7b1",
    ]
    res = registry.dataset_metrics(uuids)
    assert list == res.__class__
    assert 2 == len(res)



"""Tests for species module - name_usage methods"""
import vcr
from pygbif import species


@vcr.use_cassette("test/vcr_cassettes/test_name_usage.yaml")
def test_name_usage():
    "species.name_usage - basic test"
    res = species.name_usage(key=1)
    assert dict == res.__class__
    assert 23 == len(res)
    assert 1 == res["key"]


@vcr.use_cassette("test/vcr_cassettes/test_name_usage_paging.yaml")
def test_name_usage_paging():
    "species.name_usage - paging"
    res = species.name_usage(limit=10)
    assert dict == res.__class__
    assert 4 == len(res)
    assert 10 == len(res["results"])


@vcr.use_cassette("test/vcr_cassettes/test_name_usage_datasetkey.yaml")
def test_name_usage_datasetkey():
    "species.name_usage - datasetkey works"
    res = species.name_usage(datasetKey="d7dddbf4-2cf0-4f39-9b2a-bb099caae36c")
    assert dict == res.__class__
    assert 4 == len(res)
    assert (
        "d7dddbf4-2cf0-4f39-9b2a-bb099caae36c"
        == list(set([x["datasetKey"] for x in res["results"]]))[0]
    )
